%%
%====BAG OF FEATURES ANALYSIS====
% Modified from the Mathworks example for BoF category classification,
% https://www.mathworks.com/help/vision/examples/image-category-classification-using-bag-of-features.html

%==Natural Stimuli==

rootFolder = fullfile('Stimuli/Natural');
imgSets = imageSet(rootFolder,'recursive');
minSetCount = min([imgSets.Count]); % set the smallest amount of images in a category

%==Painterly Stimuli==

% rootFolder = fullfile('Stimuli/Painterly');
% imgSets = imageSet(rootFolder,'recursive');
% minSetCount = min([imgSets.Count]); % set the smallest amount of images in a category

%==Analysis==

imgSets = partition(imgSets, minSetCount, 'randomize'); % Use partition method to trim the set.

n = 16;
output = zeros(2*(n^2),50);
for i = 1:50
	[TrainingSets, ValidationSets] = partition(imgSets, 0.5, 'randomize');
	Fbag = bagOfFeatures(TrainingSets,'VocabularySize',100);
	CategoryClassifier = trainImageCategoryClassifier(TrainingSets, Fbag);
	[ConfMatrix_T,KnownLabelIdx_T,~] = evaluate(CategoryClassifier,TrainingSets);
	[PredictedLabelIdx_T, Score_T] = predict(CategoryClassifier, TrainingSets);
	[ConfMatrix_V,KnownLabelIdx_V,~] = evaluate(CategoryClassifier,ValidationSets);
	[PredictedLabelIdx_V, Score_V] = predict(CategoryClassifier, ValidationSets);
	output(1:16,i) = ConfMatrix_T(:);
	output(17:32,i) = ConfMatrix_T(:);
end

%%
%====ALEXNET ANALYSIS====
% Modified from the Mathworks example for CNN category classification,
% https://www.mathworks.com/help/vision/examples/image-category-classification-using-deep-learning.html

%==Natural Stimuli==

imds = imageDatastore({'Stimuli\Birds\*','Stimuli\Cats\*','Stimuli\Flowers\*','Stimuli\People\*'},'LabelSource', 'foldernames');

%==Painterly Stimuli==

% imds = imageDatastore({'Stimuli\Dali\*',
%     'Stimuli\Gerome\*',
%     'Stimuli\Monet\*',
%     'Stimuli\VanGogh\*'},'LabelSource', 'foldernames');

%==Labels==

tbl = countEachLabel(imds);
minSetCount = min(tbl{:,2}); % Determine the smallest amount of images in a category
imds = splitEachLabel(imds, minSetCount, 'randomize'); % Use splitEachLabel method to trim the set.

%==Load AlexNet==

cnnURL = 'http://www.vlfeat.org/matconvnet/models/beta16/imagenet-caffe-alex.mat';
cnnMatFile = fullfile(tempdir, 'imagenet-caffe-alex.mat');
if ~exist(cnnMatFile, 'file') % download only once
    disp('Downloading pre-trained CNN model...');
    websave(cnnMatFile, cnnURL);
end
convnet = helperImportMatConvNet(cnnMatFile);
imds.ReadFcn = @(filename)readAndPreprocessImage(filename);

%==Analysis==

n = 16;
featureLayers = {'conv1','conv2','conv3','conv4','conv5','fc6','fc7'};
output = zeros(7*(n^2),50);
for i = 1:50
	[trainingSet, testSet] = splitEachLabel(imds, 0.3, 'randomize');
	for FL = 1:7
		trainingFeatures = activations(convnet, trainingSet, featureLayers{FL}, 'MiniBatchSize', 32, 'OutputAs', 'columns');
		trainingLabels = trainingSet.Labels;
		classifier = fitcecoc(trainingFeatures, trainingLabels, 'Learners', 'Linear', 'Coding', 'onevsall', 'ObservationsIn', 'columns');
		testFeatures = activations(convnet, testSet, featureLayers{FL}, 'MiniBatchSize',32);
		predictedLabels = predict(classifier, testFeatures);
		testLabels = testSet.Labels;
		confMat = confusionmat(testLabels, predictedLabels);
		confMat = bsxfun(@rdivide,confMat,sum(confMat,2));
		bot = (FL-1)*(n^2) + 1;
		output(bot:(bot+(n^2)-1),i) = confMat(:);
	end
end



